#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 17/11/2002
################################################################################
#
# Board - Messages board functionality for Ethereal-Realms
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date processing
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handler
use Ethereal::Template;						# Template handler


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter handle
my $script;							# Script name

my %board;							# Board hash

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Capture scriptname
        $script = $cgi->url(-relative=>1);


	# Retreive hash
	$database->GetHashBoard(\%board);

	# Display hanndler
	# Complete
	unless ($script =~ /^quicklog/)
	{
		# Document header
		print $cgi->header();
		$database->DocumentGetHeader();

		PostView($database, $cgi, $param, \%board);

		# Document footer
		$database->DocumentGetFooter();	
	}

	# Stripped
	else
	{
		QuickView($database, $cgi, $param, \%board);
	}

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Post Viewer 
#
# Dirt simple functionality that exists simply to display posts in the
# weblog.

sub PostView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $data     = $database->{'HANDLE'};		# Database connection handly
	my $date;					# Date handler
	my $table;					# Table handler
	my $tmpl;					# Template handler

	my $res;					# Query results
	my $statement;					# Query handle

	my $dat;					# Date/time stamp
	my $dis;					# Display amount widget
	my $fil;					# Filtering by topic
	my $log;					# Login name
	my $nam;					# Discovered name
	my $num;					# Number to display
	my $top;					# Currently selected topic
	my $url;					# Self referencing address
	my $wid;					# Specific message

	my $sub;					# Sub information
	my $tit;					# Formatted title

	my $inl;					# Basic inline
	my $nxt;					# Next set
	my $prv;					# Previous set
	my $min;					# First in line
	my $max;					# Last in line
	my $nav;					# Navigational link

	my @opt = qw(10 20 30 40 50);			# Optional amount to display

	my %fil;					# Filter inline hash
	my %bio;					# Bio inline hash


	#####################
	# Program area

	# Instance assignment
	$date  = new Ethereal::Date();
	$table = new Ethereal::Table($cgi);
	$tmpl  = new Ethereal::Template($board);

	# Next and previous adjustment
	($min, $max) = $database->DataGet("SELECT 
		  MIN(LogIDNumber), 
		  MAX(LogIDNumber) 
		FROM Weblog;");

	# Definition
	$fil = (defined($param->{'FIL'})) ? "AND LogTopic=\"$param->{FIL}\"" : "";
	$wid = (defined($param->{'WID'})) ? "AND LogIDNumber=\"$param->{WID}\"" : "";

	# Cleanup
	$param->Cleanup('WID');

	# Parameter handling
	$param->{'NUM'} = ((defined($param->{'NUM'})) && ($param->{'NUM'} =~ /^\d+$/)) ? $param->{'NUM'} : 10;
	$param->{'TOP'} = ((defined($param->{'TOP'})) && ($param->{'TOP'} =~ /^\d+$/)) ? $param->{'TOP'} : $max;

	# Assignment
	$url = $cgi->url();
	$num = $param->{'NUM'};

	# Display prep
	# Initial title
	$tmpl->ShowMain($board->{'TagTitleWeblog'});

	# Table arrange
	$table->MakeTop("90%");

	# Find and list
	# Prepare and execute	
	$statement = $data->prepare("SELECT
		  LogIDNumber,
		  PuppetName,
		  LogTitle,
		  LogTopic,
		  LogTimestamp,
		  LogPost
		FROM Weblog
		WHERE LogIDNumber <= ?
		$fil
		$wid
		ORDER BY LogIDNumber DESC
		LIMIT $num");
	$statement->execute($param->{'TOP'});


	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Value handling
		# Assign
		$nam =  $res->{'PuppetName'};
		$top =  $res->{'LogTopic'};

		# Pull login name
		($log) = $database->DataGetPuppeteerLogin($nam);
		($dat) = $date->GetDate($res->{'LogTimestamp'});

		# Next and previous values
		$nxt = $res->{'LogIDNumber'};
		$prv = (defined($prv)) ? $prv : $res->{'LogIDNumber'};

		# Define hashes
		$bio{$nam} = (defined($bio{$nam})) ? $bio{$nam} : $param->EmbedInline('USER', $log, 'CHAR', $nam);
		$fil{$top} = (defined($fil{$top})) ? $fil{$top} : $param->EmbedInline($param->Flat(), 'FIL', $top);


		# Create display
		# Title
		$tit = $tmpl->PassSub($res->{'LogTitle'});

		# Subtitle
		$sub = "[<B><A HREF=\"$url$fil{$top}\">" . uc($top) . "</A></B>]" 
		 . " $board->{TagHeadFrom} <A HREF=\"$board->{TagLnkBio}$bio{$nam}\" TARGET=\"_new\">$nam</A> "
		 . $board->{TagHeadDate} . " $dat";


		# Display
		# Header
		$table->MakeSingle($tit);
		$table->MakeSingle($sub);
		$table->MakeBlank();

		# Post
		$table->MakeSingle($res->{'LogPost'});
		$table->MakeBlank();
	}

	# Finish query
	$statement->finish();


	# Directional handling
	# Previous and last
	$prv = ($prv != $max) ? ($prv + ($num)) : $max;
	$nxt = ($nxt != $min) ? ($nxt-1) : ($min + ($num-1));
	$prv = $param->EmbedInline($param->Flat(), 'TOP', $prv);
	$nxt = $param->EmbedInline($param->Flat(), 'TOP', $nxt);

	# First and last
	$min = $param->EmbedInline($param->Flat(), 'TOP', ($min + ($num-1)));
	$max = $param->EmbedInline($param->Flat(), 'TOP', $max);


	# Generate nav link
	# Cleaner code
	$nav = "<A HREF=\"$url\">$board->{TagReset}</A> - "
	     . "<A HREF=\"$url$max\">$board->{TagFirst}</A> " 
	     . "<A HREF=\"$url$prv\">$board->{TagPrev}</A> " 
	     . "<A HREF=\"$url$nxt\">$board->{TagNext}</A> "
	     . "<A HREF=\"$url$min\">$board->{TagLast}</A> - ";

	# Cleanup
	$param->Cleanup('NUM');

	# Generate number restrictors
	foreach $inl (@opt)
	{
		# Assign and modify
		$tit =  $board->{'TagVar'};
		$tit =~ s/MTITLE/$inl/gs;

		# Create parameter
		$inl =  $param->EmbedInline($param->Flat(), 'NUM', $inl);

		# Add to widget display
		$nav = $nav . "<A HREF=\"$url$inl\">$tit</A> ";
	}

	# Final assigment
	$tit =  $board->{'TagTitle'};
	$tit =~ s/MTITLE/&nbsp;/gs;

	# Navigation link
	$table->MakeSingle($tit);
	$table->MakeSingle("<CENTER>$nav</CENTER>");


	# Display completion
	# End table
	$table->MakeBottom();
}


#####################
# Quick Viewer 
#
# Used in conjunction with SSI as to be able to display a part of the log
# without including the rest of the interface.  Very useful for front
# pages.

sub QuickView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $data     = $database->{'HANDLE'};		# Database connection handly
	my $date;					# Date handler
	my $table;					# Table handler

	my $res;					# Query results
	my $statement;					# Query handle

	my $dat;					# Date/time stamp
	my $dis;					# Display amount widget
	my $fil;					# Filtering by topic
	my $log;					# Login name
	my $nam;					# Discovered name
	my $num;					# Number to display
	my $top;					# Currently selected topic

	my $inl;					# Inline information
	my $sub;					# Sub information

	my %fil;					# Filter inline hash
	my %bio;					# Bio inline hash


	#####################
	# Program area

	# Instance assignment
	$date  = new Ethereal::Date();
	$table = new Ethereal::Table($cgi);


	# Find and list
	# Prepare and execute	
	$statement = $data->prepare("SELECT
		  LogIDNumber,
		  PuppetName,
		  LogTitle,
		  LogTopic,
		  LogTimestamp,
		  LogPost
		FROM Weblog
		ORDER BY LogIDNumber DESC
		LIMIT 3");
	$statement->execute();

	# Display prep
	# print "<BR>\n";

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Value handling
		# Assign
		$nam =  $res->{'PuppetName'};
		$top =  $res->{'LogTopic'};

		# Pull login name
		($log) = $database->DataGetPuppeteerLogin($nam);
		($dat) = $date->GetDateShort($res->{'LogTimestamp'});

		# Define hashes
		$bio{$nam} = (defined($bio{$nam})) ? $bio{$nam} : $param->EmbedInline('USER', $log, 'CHAR', $nam);
		$fil{$top} = (defined($fil{$top})) ? $fil{$top} : $param->EmbedInline($param->Flat(), 'FIL', $top);

		# Inline parameter
		$inl = $param->EmbedInline($param->Flat(), 'WID', $res->{'LogIDNumber'});


		# Create display
		# Subtitle
		$sub = "<B><A HREF=\"$board->{TagLnkBoard}$fil{$top}\">" . uc($top) . "</A></B><BR>" 
		 . "<A HREF=\"$board->{TagLnkBoard}$inl\" TARGET=\"_new\">$res->{LogTitle}</A><BR> "
		 . "$board->{TagHeadFrom} <A HREF=\"$board->{TagLnkBio}$bio{$nam}\" TARGET=\"_new\">$nam</A> "
		 . "(" . $dat . ")";



		# Display
		print "<P>$sub</P>\n";
	}

	# Finish query
	$statement->finish();
}
